# ------------------------------------------------------------
# Class for holding an annotation definition for a test case
# ------------------------------------------------------------


class Annotation:

    def __init__(self, class_name, test_name, test_rail_ids, jira_defect_id, jira_story_id, jira_epic_id):
        self.testPackage = class_name.split('.')[-1]
        self.testClass = class_name
        self.testMethod = test_name
        self.TestRailID = test_rail_ids
        self.JIRADefect = jira_defect_id
        self.JIRAStory = jira_story_id
        self.JIRAEpic = jira_epic_id

