# ---------------------------------------------------------
# Browser Locations
# -----------------------------------------------------------

from enum import Enum


class BrowserLocations(Enum):
    Local = 1,
    Remote = 2
    
    @staticmethod
    def get_browser_location(browser_location):
        try:
            if browser_location.lower()=='remote':
                return BrowserLocations.Remote
            else:
                return BrowserLocations.Local
        except:
            return BrowserLocations.Local