from setuptools import setup, find_packages
import os

folder_path = os.path.join(os.path.dirname(__file__), 'src/accelarobotkeywords/framework/selenium/drivers/*')

setup(
    # Needed to silence warnings (and to be a worthwhile package)
    name='accelarobotkeywords',
    url='https://github.com/Accela-Inc/robotfwk',
    author='Accela',
    author_email='AutoBots@accela.onmicrosoft.com',
    # Needed for dependencies
    package_data={'accelarobotkeywords': [folder_path]},
    install_requires=['selenium', 'robotframework', 'docker', 'pyvirtualdisplay', 'pyscreenshot', 'Pillow'],
    # directories
    packages=find_packages('src'),
    package_dir={'': 'src'},
    # need to declare this entry point as a module or it will not find it at runtime.
    # py_modules=['SeleniumKeywords', 'FrameworkKeywords', 'AnnotationsListener'],
    entry_points={
        'console_scripts': [
            'RobotParallelRunner = RobotParallelRunner:main'
        ],
    },
    # *strongly* suggested for sharing
    version='0.79.9',
    # The license can be anything you like
    license='MIT',
    description='Accela robot framework keywords library',
    setup_requires=['wheel', 'twine'],
    include_package_data=True,
    zip_safe=False
)
